import {
  Box,
  styled,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Typography
} from "@mui/material"
import {CSSTransition, TransitionGroup} from 'react-transition-group'
import {useQuery} from "@tanstack/react-query"
import {useCallback, useEffect, useMemo, useRef} from "react"
import dayjs from "dayjs"

const StyledTableRow = styled(TableRow)(({ theme, tabletype }) => ({
  'tbody &:nth-of-type(odd)': {
    backgroundColor: theme.palette.sncf[tabletype].light,
  },
  'th, &:nth-of-type(even)': {
    backgroundColor: theme.palette.sncf[tabletype].dark,
  },
  // hide last border
  '&:last-child td, &:last-child th': {
    border: 0,
  },
}));

function TrainsTable({station, datetime, realtime, tableType}) {
  return <>
    <TableContainer>
      <Table>
        <TrainsTableHeader tableType={tableType} />
        <TrainsTableBody station={station} datetime={datetime} realtime={realtime} tableType={tableType} />
      </Table>
    </TableContainer>
  </>
}

function TrainsTableHeader({tableType}) {
  return <>
    <TableHead>
      <StyledTableRow tabletype={tableType}>
        <TableCell colSpan="2" fontSize={16} fontWeight="bold">Train</TableCell>
        <TableCell fontSize={16} fontWeight="bold">Heure</TableCell>
        <TableCell fontSize={16} fontWeight="bold">Destination</TableCell>
      </StyledTableRow>
    </TableHead>
  </>
}

function TrainsTableBody({station, datetime, realtime, tableType}) {
  const filterTime = useCallback((train) => {
    if (tableType === "departures")
      return dayjs(train.place.departure) >= datetime
    else
      return dayjs(train.place.arrival) >= datetime
  }, [datetime, tableType])

  const updateTrains = useCallback(() => {
    const params = {
      stopId: station.stopId,
      arriveBy: tableType === "arrivals",
      direction: "LATER",
      n: 20,
    }
    if (!realtime)
      params['time'] = datetime.format()
    const query_params = new URLSearchParams(params).toString()
    return fetch(`${process.env.REACT_APP_MOTIS_SERVER}/api/v1/stoptimes?${query_params}`)
        .then(response => response.json())
        .then(data => data.stopTimes)
        .then(data => [...data])
  }, [station.stopId, tableType, datetime, realtime])

  const trainsQuery = useQuery({
    queryKey: ['trains', station.stopId, tableType],
    queryFn: updateTrains,
    enabled: !!station.stopId,
  })
  const trains = useMemo(() => trainsQuery.data ?? [], [trainsQuery.data])

  useEffect(() => {
    if (realtime) {
      let validTrains = trains?.filter(filterTime) ?? []
      if ((trains?.length > 0 && validTrains.length < trains?.length))
        trainsQuery.refetch().then()
    }
  }, [trains, filterTime, trainsQuery, realtime])

  const nullRef = useRef(null)
  let table_rows = trains.map((train) => <CSSTransition key={train.id} timeout={500} classNames="shrink" nodeRef={nullRef}>
      <TrainRow train={train} tableType={tableType} />
    </CSSTransition>)

  return <>
    <TableBody>
      <TransitionGroup component={null}>
        {table_rows}
      </TransitionGroup>
    </TableBody>
  </>
}

function TrainRow({train, tableType}) {
  const tripQuery = useQuery({
    queryKey: ['tripId', train.tripId],
    queryFn: () => fetch(`${process.env.REACT_APP_MOTIS_SERVER}/api/v1/trip?${new URLSearchParams({tripId: train.tripId})}`)
        .then(response => response.json()),
    enabled: !!train.tripId,
  })
  const trip = tripQuery.data ?? {}
  const leg = trip.legs ? trip.legs[0] : null

  const trainType = getTrainType(train)
  const backgroundColor = getBackgroundColor(train)
  const textColor = getTextColor(train)
  const trainTypeDisplay = getTrainTypeDisplay(trainType)

  const stops = useMemo(() => leg ? [leg.from, ...leg.intermediateStops, leg.to] : [], [leg])
  const stopIndex = useMemo(() => {
    if (stops.length === 0 || train.place.stopId === undefined)
      return -1
    for (let i = 0; i < stops.length; i++) {
      const index = tableType === "departures" ? i : stops.length - 1 - i
      const stop = stops[index]
      let timeCond = tableType === "departures" ? stop.scheduledDeparture === train.place.scheduledDeparture
        : stop.scheduledArrival === train.place.scheduledArrival
      if (stop.stopId === train.place.stopId && timeCond)
        return index
    }
  }, [stops, train, tableType])
  const nextStops = tableType === "departures" ? stops.slice(stopIndex + 1) : stops.slice(0, stopIndex)

  let headline = nextStops[tableType === "departures" ? nextStops.length - 1 : 0] ?? {name: "Chargement…"}

  const canceled = false  // TODO Implémenter l'annulation
  const [delayed, prettyDelay] = getPrettyDelay(train, tableType)

  let stopsNames = nextStops.map(stopTime => stopTime?.name ?? "").join(" > ") ?? ""

  return <>
    <StyledTableRow tabletype={tableType}>
      <TableCell>
        <div>
          <Box display="flex"
               justifyContent="center"
               alignItems="center"
               textAlign="center"
               width="4em"
               height="4em"
               borderRadius="15%"
               fontWeight="bold"
               backgroundColor={backgroundColor}
               color={textColor}>
            {trainTypeDisplay}
          </Box>
        </div>
      </TableCell>
      <TableCell>
        <Box display="flex" alignItems="center" justifyContent="center" textAlign="center">
          <div>
            <div>{train.routeShortName}</div>
            <div>{train.headsign}</div>
          </div>
        </Box>
      </TableCell>
      <TableCell>
        <Box display="flex" alignItems="center" justifyContent="center">
          <Box>
            <Box fontWeight="bold" color="#FFED02" fontSize={24}>
              {getDisplayTime(train, tableType)}
            </Box>
            <Box color={delayed ? "#e86d2b" : "white"}
                 fontWeight={delayed ? "bold" : ""}>
              {prettyDelay}
            </Box>
          </Box>
        </Box>
      </TableCell>
      <TableCell>
        <Box style={{textDecoration: canceled ? 'line-through': ''}}>
          <Typography fontSize={24} fontWeight="bold" data-stop-id={headline.stopId}>{headline.name}</Typography>
          <span className="stops">{stopsNames}</span>
        </Box>
      </TableCell>
    </StyledTableRow>
  </>
}

function getTrainType(train) {
  if (train.place.stopId === undefined)
    return ""
  switch (train.place.stopId.split('_')[0]) {
    case "FR-SNCF-TGV":
    case "FR-SNCF-IC":
    case "FR-SNCF-TER":
      let trainType = train.place.stopId.split("StopPoint:OCE")[1].split("-")[0]
      switch (trainType) {
        case "Train TER":
          return "TER"
        case "INTERCITES":
          return "INTER-CITÉS"
        case "INTERCITES de nuit":
          return "INTER-CITÉS de nuit"
        default:
          return trainType
      }
    case "FR-IDFM":
      const route_split = train.routeShortName.split(" ")
      if (route_split[0] === "Bus")
        return route_split[1]
      return route_split[0]
    case "FR-GES-CTS":
      return train.routeShortName.split(" ")[1]
    case "FR-EUROSTAR":
      return "Eurostar"
    case "IT-FRA-TI":
      return "Trenitalia France"
    case "ES-RENFE":
      return "RENFE"
    case "AT-OBB":
      if (train.routeShortName?.startsWith("NJ"))
        return "NJ"
      return "ÖBB"
    case "CH-ALL":
    default:
      return train.routeShortName?.split(" ")[0]
  }
}

function getTrainTypeDisplay(trainType) {
  switch (trainType) {
    case "TGV INOUI":
      return <img src="/tgv_inoui.svg" alt="TGV INOUI" width="80%" />
    case "OUIGO":
      return <img src="/ouigo.svg" alt="OUIGO" width="80%" />
    case "ICE":
      return <img src="/ice.svg" alt="ICE" width="80%" />
    case "Lyria":
      return <img src="/lyria.svg" alt="Lyria" width="80%" />
    case "TER":
      return <img src="/ter.svg" alt="TER" width="80%" />
    case "Car TER":
      return <div><img src="/bus.svg" alt="Car" width="40%" />
        <br/>
        <img src="/ter.svg" alt="TER" width="40%" /></div>
    case "Eurostar":
      return <img src="/eurostar_mini.svg" alt="Eurostar" width="80%" />
    case "Trenitalia":
    case "Trenitalia France":
      return <img src="/trenitalia.svg" alt="Frecciarossa" width="80%" />
    case "RENFE":
      return <img src="/renfe.svg" alt="RENFE" width="80%" />
    case "NJ":
      return <img src="/nightjet.svg" alt="NightJet" width="80%" />
    default:
      return trainType
  }
}

function getBackgroundColor(train) {
  let trainType = getTrainType(train)
  switch (trainType) {
    case "OUIGO":
      return "#0096CA"
    case "Eurostar":
      return "#00286A"
    case "NJ":
      return "#272759"
    default:
      if (train.routeColor)
        return `#${train.routeColor}`
      return "#FFFFFF"
  }
}

function getTextColor(train) {
  if (train.routeTextColor)
    return `#${train.routeTextColor}`
  else {
    let trainType = getTrainType(train)
    switch (trainType) {
      case "OUIGO":
        return "#FFFFFF"
      case "TGV INOUI":
        return "#9B2743"
      case "ICE":
        return "#B4B4B4"
      case "INTER-CITÉS":
      case "INTER-CITÉS de nuit":
        return "#404042"
      default:
        return "#000000"
    }
  }
}

function getDisplayTime(train, tableType) {
  dayjs.locale('fr')
  let time = tableType === "departures" ? train.place.scheduledDeparture : train.place.scheduledArrival
  return dayjs(time).format('LT')
}

function getPrettyDelay(train, tableType) {
  if (train === undefined || !train.realTime) {
    return [false, ""]
  }
  const [scheduled, projected] = tableType === "departures" ? [train.place.scheduledDeparture, train.place.departure]
    : [train.place.scheduledArrival, train.place.arrival]
  const delay_minutes = dayjs(projected).diff(dayjs(scheduled), "minute")
  if (delay_minutes === 0)
    return [false, "À l'heure"]
  return [true, `+${delay_minutes} min`]
}

export default TrainsTable;